from Products.validation.interfaces import ivalidator

class PointEventsValidator:

    __implements__ = (ivalidator,)

    def __init__(self, name):
        self.name = name

    def __call__(self, value, *args, **kwargs):
        self.lines = {'label_placement' : [],
                      'x'               : [],
                      'label_angle'     : [],
                      'symbol_type'     : [],
                      'symbol_width'    : [],
                      'symbol_height'   : [],
                      'age'             : [],
                      'font_size'       : [],
                      'text'            : [],
                      'color'           : []}
                  
        # setup and check for item count errors:

        line_errors = []
        line_count = 1
        for event_string in value:
            label_placement = ""
            label_angle = ""
            x = ""
            symbol_type = ""
            symbol_width = ""
            symbol_height = ""
            age = ""
            font_size = ""
            text = ""
            color = ""
            try:
                items = eval('[%s]' % event_string)
            except:
                return "Mixed numeric and text input somewhere on line %d" % line_count
            if len(items) != 12:
                line_errors.append("%d" % line_count)
            else:
                label_placement,label_angle,x,symbol_type,symbol_width,symbol_height,age,font_size,text,color = items
                self.lines['label_placement'].append(label_placement)
                self.lines['label_angle'].append(label_angle)
                self.lines['x'].append(x)
                self.lines['symbol_type'].append(symbol_type)
                self.lines['symbol_width'].append(symbol_width)
                self.lines['symbol_height'].append(symbol_height)
                self.lines['age'].append(age)
                self.lines['font_size'].append(font_size)
                self.lines['text'].append(text)
                self.lines['color'].append(color)
            line_count += 1
        if len(line_errors) > 0:
            return "Wrong number of items on lines: %s" % line_errors

        # check for basic conversion errors:

        error_message = ""
        errors = None

        errors = self.checkForLabelPlacementErrors()
        if errors:
            error_message = "Label Placement (field 1) error on lines: %s" % str(errors)
            return error_message

        errors = self.checkForLabelAngleErrors()
        if errors:
            error_message = "Label Angle (field 2) error on lines: %s" % str(errors)
            return error_message

        errors = self.checkForXPositionErrors()
        if errors:
            error_message = "X Position (field 3) error on lines: %s" % str(errors)
            return error_message

        errors = self.checkForSymbolCodeErrors()
        if errors:
            error_message = "Symbol Code (field 4) error on lines: %s" % str(errors)
            return error_message

        errors = self.checkForSymbolWidthErrors()
        if errors:
            error_message = "Symbol Width (field 5) error on lines: %s" % str(errors)
            return error_message

        errors = self.checkForSymbolHeightErrors()
        if errors:
            error_message = "Symbol Height (field 6) error on lines: %s" % str(errors)
            return error_message

        errors = self.checkForTimeErrors()
        if errors:
            error_message = "Time (field 7) error on lines: %s" % str(errors)
            return error_message

        errors = self.checkForFontSizeErrors()
        if errors:
            error_message = "Text (field 8) error on lines: %s" % str(errors)
            return error_message

        errors = self.checkForTextErrors()
        if errors:
            error_message = "Text (field 9) error on lines: %s" % str(errors)
            return error_message

        errors = self.checkForColorErrors()
        if errors:
            error_message = "Color (field 10) error on lines: %s" % str(errors)
            return error_message

        return 1

    def checkForLabelPlacementErrors(self):
        # This was needed before this was a SelectColumn
	#
        #line_errors = []
        #count = 1
        #choices = ['t', 'b', 'l', 'r', 'c', 'T', 'B', 'L', 'R', 'C']
        #for label_placement in self.lines['label_placement']:
        #    if label_placement not in choices or len(label_placement) == 0 or len(label_placement) > 1:
        #        line_errors.append(count)
        #    count += 1
        #if len(line_errors) > 0:
        #    return line_errors

	# now, just pass
        return None

    def checkForLabelAngleErrors(self):
        line_errors = []
        count = 1
        for label_angle in self.lines['label_angle']:
            try:
                int(label_angle)
            except:
                line_errors.append(count)
            count += 1
        if len(line_errors) > 0:
            return line_errors
        return None

    def checkForXPositionErrors(self):
        line_errors = []
        count = 1
        for x in self.lines['x']:
            try:
                int(x)
            except:
                line_errors.append(count)
            if int(x) < 0:
                line_errors.append(count)
            count += 1
        if len(line_errors) > 0:
            return line_errors
        return None

    def checkForSymbolCodeErrors(self):
        line_errors = []
        count = 1
        choices = ['ellipse','rect',]
        for symbol_type in self.lines['symbol_type']:
            if symbol_type not in choices or len(symbol_type) == 0:
                line_errors.append(count)
            count += 1
        if len(line_errors) > 0:
            return line_errors
        return None

    def checkForSymbolWidthErrors(self):
        line_errors = []
        count = 1
        for symbol_width in self.lines['symbol_width']:
            try:
                int(symbol_width)
            except:
                line_errors.append(count)
            # don't let it be zero
            if int(symbol_width) <= 0:
                line_errors.append(count)
            count += 1
        if len(line_errors) > 0:
            return line_errors
        return None

    def checkForSymbolHeightErrors(self):
        line_errors = []
        count = 1
        for symbol_height in self.lines['symbol_height']:
            try:
                int(symbol_height)
            except:
                line_errors.append(count)
            # don't let it be zero
            if int(symbol_height) <= 0:
                line_errors.append(count)
            count += 1
        if len(line_errors) > 0:
            return line_errors
        return None

    def checkForTimeErrors(self):
        line_errors = []
        count = 1
        for age in self.lines['age']:
            try:
                float(age)
            except:
                line_errors.append(count)
            if float(age) < 0.0:
                line_errors.append(count)
            count += 1
        if len(line_errors) > 0:
            return line_errors
        return None

    def checkForFontSizeErrors(self):
        line_errors = []
        count = 1
        for font_size in self.lines['font_size']:
            try:
                int(font_size)
            except:
                line_errors.append(count)
            count += 1
        if len(line_errors) > 0:
            return line_errors
        return None

    def checkForTextErrors(self):
        # pass
        return None

    def checkForColorErrors(self):
        # pass
        return None

    # This was used when input was r,g,b
    def checkForColorRangeErrors(self):
        line_errors = []
        for i in range(len(self.lines['red'])):
            red = int(self.lines['red'][i])
            green = int(self.lines['green'][i])
            blue = int(self.lines['blue'][i])

            # must be between 0 and 255
            if red < 0 or red > 255 or green < 0 or green > 255 or blue < 0 or blue > 255:
                line_errors.append(i+1)
        if len(line_errors) > 0:
            return line_errors
        return None
